package com.example.application.views.results;

import com.example.application.data.bean.MatchForMatchdayView;
import com.example.application.data.entity.Matchday;
import com.example.application.data.entity.Player;
import com.example.application.data.entity.Season;
import com.example.application.data.service.MatchService;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.SeasonService;
import com.example.application.views.main.MainView;
import com.vaadin.flow.component.*;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.Div;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.select.Select;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.ArrayList;
import java.util.List;

@CssImport("./views/results/results-view.css")
@Route(value = "results", layout = MainView.class)
@RouteAlias(value = "", layout = MainView.class)
@PageTitle("Results")
public class ResultsView extends Div implements HasUrlParameter<String> {

    private final SeasonService seasonService;
    private final MatchdayService matchdayService;
    private final MatchService matchService;

    private String seasonParam;
    private String matchdayParam;

    private final Label invalidUrlLabel = new Label();
    private final Label matchdayViewHeader = new Label();

    private final VerticalLayout outer = new VerticalLayout();
    private final HorizontalLayout selectionLayout = new HorizontalLayout();
    private final HorizontalLayout outerMatchdaylayout = new HorizontalLayout();
    private final VerticalLayout innerMatchdayLayout = new VerticalLayout();

    private final List<Season> seasonList = new ArrayList<>();
    private final Select<Season> seasonSelect = new Select<>();

    private final List<Matchday> matchdayList = new ArrayList<>();
    private final Select<Matchday> matchdaySelect = new Select<>();

    private final Button prevButton = new Button(new Icon(VaadinIcon.ARROW_LEFT));
    private final Button nextButton = new Button(new Icon(VaadinIcon.ARROW_RIGHT));

    private final Grid<MatchForMatchdayView> grid = new Grid<>();

    public ResultsView(@Autowired SeasonService seasonService, @Autowired MatchdayService matchdayService, @Autowired MatchService matchService) {
        this.seasonService = seasonService;
        this.matchdayService = matchdayService;
        this.matchService = matchService;

        addClassName("results-view");

        configureOuterLayout();
        configureSelectionLayout();
        configureMatchdayViewLayout();
    }

    private void configureOuterLayout() {
        add(outer);
        outer.setAlignItems(FlexComponent.Alignment.CENTER);
    }

    private void configureSelectionLayout() {
        // pretty looks
        selectionLayout.setWidthFull();
        selectionLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        selectionLayout.setJustifyContentMode(FlexComponent.JustifyContentMode.END);
        selectionLayout.add(new Label("Season:"), seasonSelect, new Label("Matchday:"), matchdaySelect);

        seasonSelect.addValueChangeListener(seasonSelectValueChangeListener());
        matchdaySelect.addValueChangeListener(matchdaySelectValueChangeListener());

        // provide data
        fillSeasonSelectWithData();
    }

    private HasValue.ValueChangeListener<? super AbstractField.ComponentValueChangeEvent<Select<Matchday>, Matchday>> matchdaySelectValueChangeListener() {
        return matchdayChangeEvent -> getUI().ifPresent(ui -> {
            Season season = seasonSelect.getValue();
            Matchday matchday = matchdayChangeEvent.getValue();
            if (season != null && matchday != null) {
                String seasonParam = season.toString();
                String matchdayParam = matchday.toString();
                navigate(ui, seasonParam, matchdayParam);
            }
        });
    }

    private HasValue.ValueChangeListener<? super AbstractField.ComponentValueChangeEvent<Select<Season>, Season>> seasonSelectValueChangeListener() {
        return seasonChangeEvent -> getUI().ifPresent(ui -> {
            Season newSeason = seasonChangeEvent.getValue();
            if (newSeason != null) {
                String seasonParam = newSeason.toString();
                String matchdayParam = null;

                Matchday matchdayInNewSeason = null;
                Matchday matchdayInOldSeason = matchdaySelect.getValue();
                if (matchdayInOldSeason != null) {
                    matchdayParam = matchdayInOldSeason.toString();
                    matchdayInNewSeason = getMatchdayFromParam(matchdayParam, newSeason);
                }
                matchdayParam = matchdayInNewSeason == null ? "1" : matchdayParam;

                navigate(ui, seasonParam, matchdayParam);
            }
        });
    }

    private void fillSeasonSelectWithData() {
        seasonList.clear();
        seasonList.addAll(seasonService.getAllSeasonsSorted());
        seasonSelect.setItems(seasonList);
    }

    private void fillMatchdaySelectWithData() {
        matchdayList.clear();
        matchdayList.addAll(matchdayService.getMatchdaysForSeason(seasonSelect.getValue()));
        matchdaySelect.setItems(matchdayList);
    }

    private void configureMatchdayViewLayout() {
        outerMatchdaylayout.add(prevButton, innerMatchdayLayout, nextButton);

        matchdayViewHeader.addClassName("big_header");

        innerMatchdayLayout.setPadding(false);
        innerMatchdayLayout.add(matchdayViewHeader, grid);
        innerMatchdayLayout.setAlignItems(FlexComponent.Alignment.CENTER);
        innerMatchdayLayout.addClassName("inner_matchday_layout");

        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");
        Label headerResult = new Label("Result");
        headerResult.addClassName("column_header");

        grid.addColumn((ValueProvider<MatchForMatchdayView, Player>) MatchForMatchdayView::getPlayer1)
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");
        grid.addColumn((ValueProvider<MatchForMatchdayView, String>) match -> "vs.")
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");
        grid.addColumn((ValueProvider<MatchForMatchdayView, Player>) MatchForMatchdayView::getPlayer2)
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");
        grid.addColumn((ValueProvider<MatchForMatchdayView, String>) match -> {
            String result = match.getScore1().toString() + " : " + match.getScore2().toString();
            return result.equals("0 : 0") ? "" : result;
        })
                .setHeader(headerResult)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        grid.setWidth("36em");
        grid.setHeightByRows(true);

        grid.addClassName("matchday_grid");
    }

    private String getPrevMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) - 1);
    }

    private String getNextMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) + 1);
    }

    private boolean isMatchDayParamValid(@NonNull String matchdayParam) {
        return matchdayList.stream().anyMatch(matchday -> matchdayParam.equals(matchday.toString()));
    }

    private ComponentEventListener<ClickEvent<Button>> getButtonClickListener(Button button, String matchdayParam) {
        return buttonClickEvent -> getUI().ifPresent(ui -> navigate(ui, seasonParam, matchdayParam));
    }

    private void navigate(UI ui, String seasonParam, String matchdayParam) {
        ui.navigate(String.format("results/%s/%s/", seasonParam, matchdayParam));
    }

    private void configureMatchdayView() {
        matchdayViewHeader.setText(String.format("Matchday %s", matchdaySelect.getValue().toString()));
        grid.setItems(matchService.getMatchesForMatchdayView(matchdaySelect.getValue()));
    }

    @Override
    public void setParameter(BeforeEvent beforeEvent, @WildcardParameter String param) {
        outer.removeAll();

        if (!param.matches("^[0-9]*/[0-9]*/?$")) {
            invalidUrlLabel.setText("Invalid URL! Please provide params in the form season/matchday/");
            outer.add(invalidUrlLabel);
            return;
        }

        String[] params = param.split("/");
        seasonParam = params[0];
        matchdayParam = params[1];

        Season season = getSeasonFromParam(seasonParam);
        if (season == null) {
            invalidUrlLabel.setText(String.format("Invalid URL! Season \"%s\" does not exist in the database!", seasonParam));
            outer.add(invalidUrlLabel);
            return;
        }

        seasonSelect.setValue(season);
        fillMatchdaySelectWithData();

        Matchday matchday = getMatchdayFromParam(matchdayParam);
        if (matchday == null) {
            invalidUrlLabel.setText(String.format("Invalid URL! Matchday \"%s\" in Season \"%s\" does not exist in the database!", matchdayParam, seasonParam));
            outer.add(invalidUrlLabel);
            return;
        }

        matchdaySelect.setValue(matchday);
        configureMatchdayView();

        outer.add(selectionLayout);
        outer.add(outerMatchdaylayout);

        configureButtons();
    }

    private void configureButtons() {
        prevButton.setEnabled(isMatchDayParamValid(getPrevMatchdayParam()));
        prevButton.addClickListener(getButtonClickListener(prevButton, getPrevMatchdayParam()));

        nextButton.setEnabled(isMatchDayParamValid(getNextMatchdayParam()));
        nextButton.addClickListener(getButtonClickListener(nextButton, getNextMatchdayParam()));
    }

    @Nullable
    private Season getSeasonFromParam(@NonNull String seasonParam) {
        for (Season season : seasonList) {
            if (seasonParam.equals(season.toString())) {
                return season;
            }
        }
        return null;
    }

    @Nullable
    private Matchday getMatchdayFromParam(@NonNull String matchdayParam) {
        return getMatchdayFromParam(matchdayParam, null);
    }

    @Nullable
    private Matchday getMatchdayFromParam(@NonNull String matchdayParam, @Nullable Season season) {
        List<Matchday> matchdayList = season == null ? this.matchdayList : matchdayService.getMatchdaysForSeason(season);
        for (Matchday matchday : matchdayList) {
            if (matchdayParam.equals(matchday.toString())) {
                return matchday;
            }
        }
        return null;
    }

}
